# Introduction

Instructions for the building the salinity processing code

## REQUIREMENTS:

Linux: zlib development package gfortran4.5 or higher cmake, version 2.8 or later

_additional known OS-specific requirements_

- Ubuntu 16.04

  - zlib1g-dev

- CentOS 7

  - glibc-static
  - libgfortran-static
  - zlib-static
  - gmp-static
  - libjpeg-turbo-static
  - libstdc++-static

- Mac:

  - libX11 can be installed by XQuartz

## Building the code

CMake is used for the build system with a CMakeLists.txt file in each directory.

Here are the steps needed to build using bash, cmake and make:

1. Get a copy of the source code

  ```
  $ mkdir <salinity root directory> # e.g. $HOME/salinity
  $ cd <salinity root directory>
  $ wget https://oceandata.sci.gsfc.nasa.gov/ocssw/salinity_code.tar.gz
  $ tar -xzf salinity_code.tar.gz
  ```

2. Setup environment variables (usually in ~/.bashrc)

  ```
  $ export CC=gcc
  $ export CXX=g++
  $ export FC=gfortran
  $ export OCSSWROOT=$HOME/salinity
  $ export OCSSW_DEBUG=1  #set to 0 if not debugging
  $ source $OCSSWROOT/OCSSW_bash.env
  ```

3. Build and install the third party libraries

  ```
  $ cd $OCSSWROOT/opt
  $ wget https://oceandata.sci.gsfc.nasa.gov/ocssw/opt-src.tar
  $ tar -xvf opt-src.tar
  ```

  ```
  $ cd $OCSSWROOT/opt/src
  $ export OCSSW_DEBUG=0 #probably do not want to debug these
  $ ./BuildIt
  ```
  Note: The HDF4 build requires the Fortran API, and only works if
  statically built.  The following changes are necessary to the hdf4 "BuildIt" script:
  ```
--- opt/src/hdf4/BuildIt        
+++ opt.salinity/src/hdf4/BuildIt
@@ -38,8 +38,8 @@
 #
 #cmake -DCMAKE_INSTALL_PREFIX=$LIB3_DIR -DHDF4_BUILD_FORTRAN=false -DH4_JPEGLIB_HEADER=$LIB3_DIR/include/jpeglib.h -DH4_ZLIB_HEADER=$LIB3_DIR/include/zlib.h -DBUILD_SHARED_LIBS=1 ..
 #
-./configure --prefix=$installdir --enable-fortran --disable-netcdf
-#./configure --prefix=$installdir --enable-shared --disable-fortran --disable-netcdf
+#./configure --prefix=$installdir --enable-fortran --disable-netcdf
+./configure --prefix=$installdir --disable-shared --enable-fortran --disable-netcdf
 if [ $? -ne 0 ]; then
     echo "***** configure failed *****"
     exit 1

  ```
4. Create build directory and run cmake

  CMake allows for out-of-source-tree builds. Make a directory for this and run cmake within it, e.g.:

  ```
  $ cd <salinity root directory>
  $ mkdir build
  $ cd build
  $ cmake ..
  ```

5. Build and install the binaries

  ```
  $ export OCSSW_DEBUG=1 #could be set to 0 if not debugging
  $ make -j 20 install
  ```
